package com.xjrsoft.module.student.controller;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.xjrsoft.common.constant.GlobalConstant;
import com.baomidou.mybatisplus.core.toolkit.StringPool;
import com.xjrsoft.common.page.ConventPage;
import com.xjrsoft.common.page.PageOutput;
import com.xjrsoft.common.model.result.RT;
import com.xjrsoft.common.utils.VoToColumnUtil;
import com.xjrsoft.module.student.dto.AddBaseStudentClassCadreDto;
import com.xjrsoft.module.student.dto.UpdateBaseStudentClassCadreDto;
import cn.dev33.satoken.annotation.SaCheckPermission;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.support.ExcelTypeEnum;
import org.springframework.http.ResponseEntity;
import java.io.ByteArrayOutputStream;
import java.util.ArrayList;

import com.xjrsoft.module.student.dto.BaseStudentClassCadrePageDto;
import com.xjrsoft.module.student.entity.BaseStudentClassCadre;
import com.xjrsoft.module.student.service.IBaseStudentClassCadreService;
import com.xjrsoft.module.student.vo.BaseStudentClassCadrePageVo;

import com.xjrsoft.module.student.vo.BaseStudentClassCadreVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import javax.validation.constraints.NotNull;
import java.util.List;

/**
* @title: 学生班级干部管理
* @Author dzx
* @Date: 2023-11-10
* @Version 1.0
*/
@RestController
@RequestMapping("/student" + "/basestudentclasscadre")
@Api(value = "/student"  + "/basestudentclasscadre",tags = "学生班级干部管理代码")
@AllArgsConstructor
public class BaseStudentClassCadreController {


    private final IBaseStudentClassCadreService baseStudentClassCadreService;

    @GetMapping(value = "/page")
    @ApiOperation(value="学生班级干部管理列表(分页)")
    @SaCheckPermission("basestudentclasscadre:detail")
    public RT<PageOutput<BaseStudentClassCadrePageVo>> page(@Valid BaseStudentClassCadrePageDto dto){

        LambdaQueryWrapper<BaseStudentClassCadre> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper
                    .orderByDesc(BaseStudentClassCadre::getId)
                .select(BaseStudentClassCadre.class,x -> VoToColumnUtil.fieldsToColumns(BaseStudentClassCadrePageVo.class).contains(x.getProperty()));
        IPage<BaseStudentClassCadre> page = baseStudentClassCadreService.page(ConventPage.getPage(dto), queryWrapper);
        PageOutput<BaseStudentClassCadrePageVo> pageOutput = ConventPage.getPageOutput(page, BaseStudentClassCadrePageVo.class);
        return RT.ok(pageOutput);
    }

    @GetMapping(value = "/info")
    @ApiOperation(value="根据id查询学生班级干部管理信息")
    @SaCheckPermission("basestudentclasscadre:detail")
    public RT<BaseStudentClassCadreVo> info(@RequestParam Long id){
        BaseStudentClassCadre baseStudentClassCadre = baseStudentClassCadreService.getById(id);
        if (baseStudentClassCadre == null) {
           return RT.error("找不到此数据！");
        }
        return RT.ok(BeanUtil.toBean(baseStudentClassCadre, BaseStudentClassCadreVo.class));
    }


    @PostMapping
    @ApiOperation(value = "新增学生班级干部管理")
    @SaCheckPermission("basestudentclasscadre:add")
    public RT<Boolean> add(@Valid @RequestBody AddBaseStudentClassCadreDto dto){
        BaseStudentClassCadre baseStudentClassCadre = BeanUtil.toBean(dto, BaseStudentClassCadre.class);
        boolean isSuccess = baseStudentClassCadreService.save(baseStudentClassCadre);
    return RT.ok(isSuccess);
    }

    @PutMapping
    @ApiOperation(value = "修改学生班级干部管理")
    @SaCheckPermission("basestudentclasscadre:edit")
    public RT<Boolean> update(@Valid @RequestBody UpdateBaseStudentClassCadreDto dto){

        BaseStudentClassCadre baseStudentClassCadre = BeanUtil.toBean(dto, BaseStudentClassCadre.class);
        return RT.ok(baseStudentClassCadreService.updateById(baseStudentClassCadre));

    }

    @DeleteMapping
    @ApiOperation(value = "删除学生班级干部管理")
    @SaCheckPermission("basestudentclasscadre:delete")
    public RT<Boolean> delete(@Valid @RequestBody List<Long> ids){
        return RT.ok(baseStudentClassCadreService.removeBatchByIds(ids));

    }

    @GetMapping("/export")
    @ApiOperation(value = "导出")
    public ResponseEntity<byte[]> exportData(@Valid BaseStudentClassCadrePageDto dto, @RequestParam(defaultValue = "false") Boolean isTemplate) {
        List<BaseStudentClassCadrePageVo> customerList = isTemplate != null && isTemplate ? new ArrayList<>() : ((PageOutput<BaseStudentClassCadrePageVo>) page(dto).getData()).getList();
        ByteArrayOutputStream bot = new ByteArrayOutputStream();
        EasyExcel.write(bot, BaseStudentClassCadrePageVo.class).automaticMergeHead(false).excelType(ExcelTypeEnum.XLSX).sheet().doWrite(customerList);

        return RT.fileStream(bot.toByteArray(), "BaseStudentClassCadre" + ExcelTypeEnum.XLSX.getValue());
    }
}