package com.xjrsoft.module.student.controller;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.xjrsoft.common.constant.GlobalConstant;
import com.baomidou.mybatisplus.core.toolkit.StringPool;
import com.xjrsoft.common.page.ConventPage;
import com.xjrsoft.common.page.PageOutput;
import com.xjrsoft.common.model.result.RT;
import com.xjrsoft.common.utils.VoToColumnUtil;
import com.xjrsoft.module.student.dto.AddBaseStudentSchoolCadreDto;
import com.xjrsoft.module.student.dto.UpdateBaseStudentSchoolCadreDto;
import cn.dev33.satoken.annotation.SaCheckPermission;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.support.ExcelTypeEnum;
import org.springframework.http.ResponseEntity;
import java.io.ByteArrayOutputStream;
import java.util.ArrayList;

import com.xjrsoft.module.student.dto.BaseStudentSchoolCadrePageDto;
import com.xjrsoft.module.student.entity.BaseStudentSchoolCadre;
import com.xjrsoft.module.student.service.IBaseStudentSchoolCadreService;
import com.xjrsoft.module.student.vo.BaseStudentSchoolCadrePageVo;

import com.xjrsoft.module.student.vo.BaseStudentSchoolCadreVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import javax.validation.constraints.NotNull;
import java.util.List;

/**
* @title: 学生校级干部管理
* @Author dzx
* @Date: 2023-11-10
* @Version 1.0
*/
@RestController
@RequestMapping("/student" + "/basestudentschoolcadre")
@Api(value = "/student"  + "/basestudentschoolcadre",tags = "学生校级干部管理代码")
@AllArgsConstructor
public class BaseStudentSchoolCadreController {


    private final IBaseStudentSchoolCadreService baseStudentSchoolCadreService;

    @GetMapping(value = "/page")
    @ApiOperation(value="学生校级干部管理列表(分页)")
    @SaCheckPermission("basestudentschoolcadre:detail")
    public RT<PageOutput<BaseStudentSchoolCadrePageVo>> page(@Valid BaseStudentSchoolCadrePageDto dto){

        LambdaQueryWrapper<BaseStudentSchoolCadre> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper
                    .orderByDesc(BaseStudentSchoolCadre::getId)
                .select(BaseStudentSchoolCadre.class,x -> VoToColumnUtil.fieldsToColumns(BaseStudentSchoolCadrePageVo.class).contains(x.getProperty()));
        IPage<BaseStudentSchoolCadre> page = baseStudentSchoolCadreService.page(ConventPage.getPage(dto), queryWrapper);
        PageOutput<BaseStudentSchoolCadrePageVo> pageOutput = ConventPage.getPageOutput(page, BaseStudentSchoolCadrePageVo.class);
        return RT.ok(pageOutput);
    }

    @GetMapping(value = "/info")
    @ApiOperation(value="根据id查询学生校级干部管理信息")
    @SaCheckPermission("basestudentschoolcadre:detail")
    public RT<BaseStudentSchoolCadreVo> info(@RequestParam Long id){
        BaseStudentSchoolCadre baseStudentSchoolCadre = baseStudentSchoolCadreService.getById(id);
        if (baseStudentSchoolCadre == null) {
           return RT.error("找不到此数据！");
        }
        return RT.ok(BeanUtil.toBean(baseStudentSchoolCadre, BaseStudentSchoolCadreVo.class));
    }


    @PostMapping
    @ApiOperation(value = "新增学生校级干部管理")
    @SaCheckPermission("basestudentschoolcadre:add")
    public RT<Boolean> add(@Valid @RequestBody AddBaseStudentSchoolCadreDto dto){
        BaseStudentSchoolCadre baseStudentSchoolCadre = BeanUtil.toBean(dto, BaseStudentSchoolCadre.class);
        boolean isSuccess = baseStudentSchoolCadreService.save(baseStudentSchoolCadre);
    return RT.ok(isSuccess);
    }

    @PutMapping
    @ApiOperation(value = "修改学生校级干部管理")
    @SaCheckPermission("basestudentschoolcadre:edit")
    public RT<Boolean> update(@Valid @RequestBody UpdateBaseStudentSchoolCadreDto dto){

        BaseStudentSchoolCadre baseStudentSchoolCadre = BeanUtil.toBean(dto, BaseStudentSchoolCadre.class);
        return RT.ok(baseStudentSchoolCadreService.updateById(baseStudentSchoolCadre));

    }

    @DeleteMapping
    @ApiOperation(value = "删除学生校级干部管理")
    @SaCheckPermission("basestudentschoolcadre:delete")
    public RT<Boolean> delete(@Valid @RequestBody List<Long> ids){
        return RT.ok(baseStudentSchoolCadreService.removeBatchByIds(ids));

    }

    @GetMapping("/export")
    @ApiOperation(value = "导出")
    public ResponseEntity<byte[]> exportData(@Valid BaseStudentSchoolCadrePageDto dto, @RequestParam(defaultValue = "false") Boolean isTemplate) {
        List<BaseStudentSchoolCadrePageVo> customerList = isTemplate != null && isTemplate ? new ArrayList<>() : ((PageOutput<BaseStudentSchoolCadrePageVo>) page(dto).getData()).getList();
        ByteArrayOutputStream bot = new ByteArrayOutputStream();
        EasyExcel.write(bot, BaseStudentSchoolCadrePageVo.class).automaticMergeHead(false).excelType(ExcelTypeEnum.XLSX).sheet().doWrite(customerList);

        return RT.fileStream(bot.toByteArray(), "BaseStudentSchoolCadre" + ExcelTypeEnum.XLSX.getValue());
    }
}