package com.xjrsoft.module.system.controller;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.xjrsoft.common.constant.GlobalConstant;
import com.baomidou.mybatisplus.core.toolkit.StringPool;
import com.xjrsoft.common.page.ConventPage;
import com.xjrsoft.common.page.PageOutput;
import com.xjrsoft.common.model.result.RT;
import com.xjrsoft.common.utils.VoToColumnUtil;
import com.xjrsoft.module.system.dto.AddXjrMenuQuickDto;
import com.xjrsoft.module.system.dto.UpdateXjrMenuQuickDto;
import cn.dev33.satoken.annotation.SaCheckPermission;

import com.xjrsoft.module.system.dto.XjrMenuQuickListDto;
import com.xjrsoft.module.system.entity.XjrMenuQuick;
import com.xjrsoft.module.system.service.IXjrMenuQuickService;
import com.xjrsoft.module.system.vo.XjrMenuQuickListVo;

import com.xjrsoft.module.system.vo.XjrMenuQuickVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import javax.validation.constraints.NotNull;
import java.util.List;

/**
* @title: 快捷菜单
* @Author fanxp
* @Date: 2023-11-11
* @Version 1.0
*/
@RestController
@RequestMapping("/system" + "/xjrmenuquick")
@Api(value = "/system"  + "/xjrmenuquick",tags = "快捷菜单代码")
@AllArgsConstructor
public class XjrMenuQuickController {


    private final IXjrMenuQuickService xjrMenuQuickService;

    @GetMapping(value = "/list")
    @ApiOperation(value="快捷菜单列表(不分页)")
    @SaCheckPermission("xjrmenuquick:detail")
    public RT<List<XjrMenuQuickListVo>> list(@Valid XjrMenuQuickListDto dto){

        LambdaQueryWrapper<XjrMenuQuick> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper
                    .orderByDesc(XjrMenuQuick::getId)
                    .select(XjrMenuQuick.class,x -> VoToColumnUtil.fieldsToColumns(XjrMenuQuickListVo.class).contains(x.getProperty()));

        List<XjrMenuQuick> list = xjrMenuQuickService.list(queryWrapper);
        List<XjrMenuQuickListVo> listVos = BeanUtil.copyToList(list, XjrMenuQuickListVo.class);
        return RT.ok(listVos);
    }

    @GetMapping(value = "/info")
    @ApiOperation(value="根据id查询快捷菜单信息")
    @SaCheckPermission("xjrmenuquick:detail")
    public RT<XjrMenuQuickVo> info(@RequestParam Long id){
        XjrMenuQuick xjrMenuQuick = xjrMenuQuickService.getById(id);
        if (xjrMenuQuick == null) {
           return RT.error("找不到此数据！");
        }
        return RT.ok(BeanUtil.toBean(xjrMenuQuick, XjrMenuQuickVo.class));
    }


    @PostMapping
    @ApiOperation(value = "新增快捷菜单")
    @SaCheckPermission("xjrmenuquick:add")
    public RT<Boolean> add(@Valid @RequestBody AddXjrMenuQuickDto dto){
        XjrMenuQuick xjrMenuQuick = BeanUtil.toBean(dto, XjrMenuQuick.class);
        boolean isSuccess = xjrMenuQuickService.save(xjrMenuQuick);
    return RT.ok(isSuccess);
    }

    @PutMapping
    @ApiOperation(value = "修改快捷菜单")
    @SaCheckPermission("xjrmenuquick:edit")
    public RT<Boolean> update(@Valid @RequestBody UpdateXjrMenuQuickDto dto){

        XjrMenuQuick xjrMenuQuick = BeanUtil.toBean(dto, XjrMenuQuick.class);
        return RT.ok(xjrMenuQuickService.updateById(xjrMenuQuick));

    }

    @DeleteMapping
    @ApiOperation(value = "删除快捷菜单")
    @SaCheckPermission("xjrmenuquick:delete")
    public RT<Boolean> delete(@Valid @RequestBody List<Long> ids){
        return RT.ok(xjrMenuQuickService.removeBatchByIds(ids));

    }

}