package com.xjrsoft.module.system.service.impl;

import cn.dev33.satoken.stp.StpUtil;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.github.yulichang.base.MPJBaseServiceImpl;
import com.github.yulichang.toolkit.MPJWrappers;
import com.xjrsoft.common.constant.GlobalConstant;
import com.xjrsoft.common.enums.AuthorizeType;
import com.xjrsoft.common.enums.EnabledMark;
import com.xjrsoft.common.model.tree.ITreeNode;
import com.xjrsoft.common.utils.TreeUtil;
import com.xjrsoft.common.utils.VoToColumnUtil;
import com.xjrsoft.module.organization.entity.UserRoleRelation;
import com.xjrsoft.module.organization.mapper.UserRoleRelationMapper;
import com.xjrsoft.module.system.dto.MenuTreeDto;
import com.xjrsoft.module.system.entity.Authorize;
import com.xjrsoft.module.system.entity.Menu;
import com.xjrsoft.module.system.entity.Subsystem;
import com.xjrsoft.module.system.mapper.AuthorizeMapper;
import com.xjrsoft.module.system.mapper.MenuMapper;
import com.xjrsoft.module.system.service.IMenuService;
import com.xjrsoft.module.system.vo.MenuServeVo;
import com.xjrsoft.module.system.vo.MenuTreeVo;
import com.xjrsoft.module.system.vo.MenuVo;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 用户 服务实现类
 * </p>
 *
 * @author tzx
 * @since 2022-03-16
 */
@Service
@AllArgsConstructor
public class MenuServiceImpl extends MPJBaseServiceImpl<MenuMapper, Menu> implements IMenuService {

    private final UserRoleRelationMapper userRoleRelationMapper;

    private final AuthorizeMapper authorizeMapper;

    @Override
    public List<MenuVo> getAuthMenuList(MenuTreeDto dto) {
        List<UserRoleRelation> relations = userRoleRelationMapper.selectList(Wrappers.lambdaQuery(UserRoleRelation.class)
                .select(UserRoleRelation::getRoleId)
                .eq(UserRoleRelation::getUserId, StpUtil.getLoginIdAsLong()));
        if (CollectionUtils.isEmpty(relations)) {
            return new ArrayList<>();
        }
        List<Long> roleIds = relations.stream().map(UserRoleRelation::getRoleId).collect(Collectors.toList());
        List<Long> authMenuIdList = null;
        // 非超级管理员权限过滤
        if (!roleIds.contains(GlobalConstant.SUPER_ADMIN_ROLE_ID)) {
            List<Authorize> authorizeList = authorizeMapper.selectList(Wrappers.<Authorize>lambdaQuery()
                    .eq(Authorize::getAuthorizeType, AuthorizeType.MENU.getCode())
                    .in(Authorize::getRoleId, roleIds));
            if (CollectionUtils.isEmpty(authorizeList)) {
                return new ArrayList<>();
            }
            authMenuIdList = authorizeList.stream().map(Authorize::getObjectId).collect(Collectors.toList());
        }

        return this.selectJoinList(MenuVo.class,
                MPJWrappers.<Menu>lambdaJoin()
                        .like(StrUtil.isNotBlank(dto.getTitle()), Menu::getTitle, dto.getTitle())
                        .like(StrUtil.isNotBlank(dto.getCode()), Menu::getCode, dto.getCode())
                        .like(StrUtil.isNotBlank(dto.getName()), Menu::getName, dto.getName())
                        //                .like(ObjectUtil.isNotEmpty(dto.getEnabledMark()), Menu::getEnabledMark, dto.getEnabledMark())
                        .in(CollectionUtils.isNotEmpty(authMenuIdList), Menu::getId, authMenuIdList)
                        .select(Menu::getId)
                        .selectAs(Subsystem::getName, MenuVo::getSystemName)
                        .select(Menu.class, x -> VoToColumnUtil.fieldsToColumns(MenuTreeVo.class).contains(x.getProperty()))
                        .leftJoin(Subsystem.class, Subsystem::getId, Menu::getSystemId)
                        .eq(Menu::getEnabledMark, EnabledMark.ENABLED.getCode())
                        .orderByAsc(Menu::getSortCode));
    }

    /**
     * 查询用户菜单服务
     */
    @Override
    public List<MenuServeVo> getAuthMenuServe() {
        List<MenuVo> menuVos = getAuthMenuList(new MenuTreeDto());

        Map<Long, MenuServeVo> menuVoMap = new HashMap<>();
        List<MenuVo> notParents = new ArrayList<>();

        menuVos.forEach((node) -> {
            Long parentId = node.getParentId();

            //如果在List里面 找不到当前节点的父节点，则当前节点为顶级节点
            if (menuVos.stream().noneMatch(x -> StrUtil.equals(x.getId().toString(), String.valueOf(parentId)))) {
                if (!menuVoMap.containsKey(node.getSystemId())) {
                    MenuServeVo ms = new MenuServeVo();
                    ms.setId(node.getSystemId());
                    ms.setName(node.getSystemName());
                    ms.setMenuVoList(new ArrayList<>());
                    ms.getMenuVoList().add(node);
                    menuVoMap.put(node.getSystemId(),ms);
                }
                notParents.add(node);
            }

            if (!StrUtil.isEmptyIfStr(parentId) || !StrUtil.equals(String.valueOf(parentId), String.valueOf(GlobalConstant.FIRST_NODE_VALUE))) {
                menuVos.stream().filter(x -> StrUtil.equals(x.getId().toString(), String.valueOf(parentId)))
                        .findAny()
                        .ifPresent(parentNode -> {
                            System.out.println();
                        });
            }
        });
        return new ArrayList<>(menuVoMap.values());
    }
}
